@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 # Copyright (c) 2010-2014 openpyxl
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -22,34 +23,31 @@
 # @author: see AUTHORS file
 
 """Reader for a single worksheet."""
-
-# Python stdlib imports
-from warnings import warn
+from io import BytesIO, StringIO
 
 # compatibility imports
-from openpyxl.shared.compat import BytesIO
-from openpyxl.shared.xmltools import iterparse
+from openpyxl.xml.functions import iterparse
 
 # package imports
-from openpyxl.shared.xmltools import safe_iterator
-from openpyxl.cell import (
-    Cell,
-    coordinate_from_string,
-    get_column_letter,
-    column_index_from_string
-    )
+from openpyxl.cell import get_column_letter
+from openpyxl.cell import Cell
 from openpyxl.worksheet import Worksheet, ColumnDimension, RowDimension
-from openpyxl.shared.ooxml import SHEET_MAIN_NS
-from openpyxl.style import Color
-from openpyxl.styles.formatting import ConditionalFormatting
+from openpyxl.worksheet.iter_worksheet import IterableWorksheet
+from openpyxl.xml.constants import SHEET_MAIN_NS
+from openpyxl.xml.functions import safe_iterator
+from openpyxl.styles import Color
+from openpyxl.styles import colors
+from openpyxl.formatting import ConditionalFormatting
+from openpyxl.worksheet.page import PageMargins
+
 
 def _get_xml_iter(xml_source):
-
     if not hasattr(xml_source, 'read'):
-        if hasattr(xml_source, 'decode'):
-            return BytesIO(xml_source)
-        else:
-            return BytesIO(xml_source.encode('utf-8'))
+        try:
+            xml_source = xml_source.encode("utf-8")
+        except (AttributeError, UnicodeDecodeError):
+            pass
+        return BytesIO(xml_source)
     else:
         try:
             xml_source.seek(0)
@@ -58,101 +56,70 @@ def _get_xml_iter(xml_source):
         return xml_source
 
 
-def read_dimension(xml_source):
-    min_row = min_col =  max_row = max_col = None
-    source = _get_xml_iter(xml_source)
-    it = iterparse(source)
-    for event, el in it:
-        if el.tag == '{%s}dimension' % SHEET_MAIN_NS:
-            dim = el.get("ref")
-            if ':' in dim:
-                start, stop = dim.split(':')
-            else:
-                start = stop = dim
-            min_col, min_row = coordinate_from_string(start)
-            max_col, max_row = coordinate_from_string(stop)
-            return min_col, min_row, max_col, max_row
-
-        elif el.tag == '{%s}row' % SHEET_MAIN_NS:
-            row = el.get("r")
-            if min_row is None:
-                min_row = int(row)
-            span = el.get("spans", "")
-            if ":" in span:
-                start, stop = span.split(":")
-                if min_col is None:
-                    min_col = int(start)
-                    max_col = int(stop)
-                else:
-                    min_col = min(min_col, int(start))
-                    max_col = max(max_col, int(stop))
-
-        elif el.tag == '{%s}c' % SHEET_MAIN_NS:
-            coord = el.get('r')
-            column_str, row = coordinate_from_string(coord)
-            column = column_index_from_string(column_str)
-            if min_col is None:
-                min_col = column
-            else:
-                min_col = min(column, min_col)
-            if max_col is None:
-                max_col = column
-            else:
-                max_col = max(column, max_col)
-        el.clear()
-    max_row = int(row)
-    warn("Unsized worksheet")
-    return get_column_letter(min_col), min_row, get_column_letter(max_col),  max_row
-
-
 class WorkSheetParser(object):
 
-    def __init__(self, ws, xml_source, string_table, style_table, color_index=None):
+    COL_TAG = '{%s}col' % SHEET_MAIN_NS
+    ROW_TAG = '{%s}row' % SHEET_MAIN_NS
+    CELL_TAG = '{%s}c' % SHEET_MAIN_NS
+    VALUE_TAG = '{%s}v' % SHEET_MAIN_NS
+    FORMULA_TAG = '{%s}f' % SHEET_MAIN_NS
+    MERGE_TAG = '{%s}mergeCell' % SHEET_MAIN_NS
+
+    def __init__(self, ws, xml_source, shared_strings, style_table, color_index=None):
         self.ws = ws
         self.source = xml_source
-        self.string_table = string_table
+        self.shared_strings = shared_strings
         self.style_table = style_table
         self.color_index = color_index
         self.guess_types = ws.parent._guess_types
         self.data_only = ws.parent.data_only
 
     def parse(self):
-        stream = _get_xml_iter(self.source)
-        it = iterparse(stream)
-
         dispatcher = {
-            '{%s}c' % SHEET_MAIN_NS: self.parse_cell,
             '{%s}mergeCells' % SHEET_MAIN_NS: self.parse_merge,
-            '{%s}cols' % SHEET_MAIN_NS: self.parse_column_dimensions,
-            '{%s}sheetData' % SHEET_MAIN_NS: self.parse_row_dimensions,
+            '{%s}col' % SHEET_MAIN_NS: self.parse_column_dimensions,
+            '{%s}row' % SHEET_MAIN_NS: self.parse_row_dimensions,
             '{%s}printOptions' % SHEET_MAIN_NS: self.parse_print_options,
             '{%s}pageMargins' % SHEET_MAIN_NS: self.parse_margins,
             '{%s}pageSetup' % SHEET_MAIN_NS: self.parse_page_setup,
             '{%s}headerFooter' % SHEET_MAIN_NS: self.parse_header_footer,
-            '{%s}conditionalFormatting' % SHEET_MAIN_NS: self.parser_conditional_formatting
+            '{%s}conditionalFormatting' % SHEET_MAIN_NS: self.parser_conditional_formatting,
+            '{%s}autoFilter' % SHEET_MAIN_NS: self.parse_auto_filter
                       }
-        for event, element in it:
+        tags = dispatcher.keys()
+        stream = _get_xml_iter(self.source)
+        it = iterparse(stream, tag=tags)
+
+        for _, element in it:
             tag_name = element.tag
             if tag_name in dispatcher:
                 dispatcher[tag_name](element)
                 element.clear()
 
+        # Handle parsed conditional formatting rules together.
+        if len(self.ws.conditional_formatting.parse_rules):
+            self.ws.conditional_formatting.update(self.ws.conditional_formatting.parse_rules)
 
     def parse_cell(self, element):
-        value = element.findtext('{%s}v' % SHEET_MAIN_NS)
-        formula = element.find('{%s}f' % SHEET_MAIN_NS)
+        value = element.findtext(self.VALUE_TAG)
+        formula = element.find(self.FORMULA_TAG)
 
         coordinate = element.get('r')
         style_id = element.get('s')
         if style_id is not None:
             self.ws._styles[coordinate] = self.style_table.get(int(style_id))
 
-        if value is not None:
+        if value is not None and value is not '':
+            cell = self.ws[coordinate]
             data_type = element.get('t', 'n')
             if data_type == Cell.TYPE_STRING:
-                value = self.string_table.get(int(value))
-            if data_type == Cell.TYPE_BOOL:
+                value = self.shared_strings[int(value)]
+            elif data_type == Cell.TYPE_BOOL:
                 value = bool(int(value))
+            elif data_type == 'n':
+                cell._cast_numeric(value)
+                if self.data_only or formula is None:
+                    return
             if formula is not None and not self.data_only:
                 if formula.text:
                     value = "=" + formula.text
@@ -166,48 +133,40 @@ class WorkSheetParser(object):
                     if formula.get('ref'):  # Range for shared formulas
                         self.ws.formula_attributes[coordinate]['ref'] = formula.get('ref')
             if not self.guess_types and formula is None:
-                self.ws.cell(coordinate).set_explicit_value(value=value, data_type=data_type)
+                cell.set_explicit_value(value=value, data_type=data_type)
             else:
-                self.ws.cell(coordinate).value = value
-
+                cell.value = value
 
     def parse_merge(self, element):
         for mergeCell in safe_iterator(element, ('{%s}mergeCell' % SHEET_MAIN_NS)):
             self.ws.merge_cells(mergeCell.get('ref'))
 
+    def parse_column_dimensions(self, col):
+        min = int(col.get('min')) if col.get('min') else 1
+        max = int(col.get('max')) if col.get('max') else 1
+        # Ignore ranges that go up to the max column 16384.  Columns need to be extended to handle
+        # ranges without creating an entry for every single one.
+        if max != 16384:
+            for colId in range(min, max + 1):
+                column = get_column_letter(colId)
+                attrs = dict(col.items())
+                attrs['index'] = column
+                if column not in self.ws.column_dimensions:
+                    dim = ColumnDimension(**attrs)
+                    self.ws.column_dimensions[column] = dim
+                    if dim.style is not None:
+                        self.ws._styles[column] = self.style_table.get(dim.style)
 
-    def parse_column_dimensions(self, element):
-        for col in safe_iterator(element, '{%s}col' % SHEET_MAIN_NS):
-            min = int(col.get('min')) if col.get('min') else 1
-            max = int(col.get('max')) if col.get('max') else 1
-            # Ignore ranges that go up to the max column 16384.  Columns need to be extended to handle
-            # ranges without creating an entry for every single one.
-            if max != 16384:
-                for colId in range(min, max + 1):
-                    column = get_column_letter(colId)
-                    width = col.get("width")
-                    auto_size = col.get('bestFit') == '1'
-                    visible = col.get('hidden') != '1'
-                    outline = col.get('outlineLevel') or 0
-                    collapsed = col.get('collapsed') == '1'
-                    style_index =  self.style_table.get(int(col.get('style', 0)))
-                    if column not in self.ws.column_dimensions:
-                        new_dim = ColumnDimension(index=column,
-                                                  width=width, auto_size=auto_size,
-                                                  visible=visible, outline_level=outline,
-                                                  collapsed=collapsed, style_index=style_index)
-                        self.ws.column_dimensions[column] = new_dim
-
-
-    def parse_row_dimensions(self, element):
-        for row in safe_iterator(element, '{%s}row' % SHEET_MAIN_NS):
-            rowId = int(row.get('r'))
-            if rowId not in self.ws.row_dimensions:
-                self.ws.row_dimensions[rowId] = RowDimension(rowId)
-            ht = row.get('ht')
-            if ht is not None:
-                self.ws.row_dimensions[rowId].height = float(ht)
-
+    def parse_row_dimensions(self, row):
+        rowId = int(row.get('r'))
+        ht = row.get('ht')
+        if rowId not in self.ws.row_dimensions:
+            self.ws.row_dimensions[rowId] = RowDimension(rowId, height=ht)
+        style_index = row.get('s')
+        if row.get('customFormat') and style_index:
+            self.ws._styles[rowId] = self.style_table.get(int(style_index))
+        for cell in safe_iterator(row, self.CELL_TAG):
+            self.parse_cell(cell)
 
     def parse_print_options(self, element):
         hc = element.get('horizontalCentered')
@@ -217,13 +176,13 @@ class WorkSheetParser(object):
         if vc is not None:
             self.ws.page_setup.verticalCentered = vc
 
-
     def parse_margins(self, element):
-        for key in ("left", "right", "top", "bottom", "header", "footer"):
-            value = element.get(key)
-            if value is not None:
-                setattr(self.ws.page_margins, key, float(value))
-
+        margins = dict(element.items())
+        self.page_margins = PageMargins(**margins)
+        #for key in ("left", "right", "top", "bottom", "header", "footer"):
+            #value = element.get(key)
+            #if value is not None:
+                #setattr(self.ws.page_margins, key, float(value))
 
     def parse_page_setup(self, element):
         for key in ("orientation", "paperSize", "scale", "fitToPage",
@@ -233,7 +192,6 @@ class WorkSheetParser(object):
             if value is not None:
                 setattr(self.ws.page_setup, key, value)
 
-
     def parse_header_footer(self, element):
         oddHeader = element.find('{%s}oddHeader' % SHEET_MAIN_NS)
         if oddHeader is not None and oddHeader.text is not None:
@@ -242,16 +200,15 @@ class WorkSheetParser(object):
         if oddFooter is not None and oddFooter.text is not None:
             self.ws.header_footer.setFooter(oddFooter.text)
 
-
     def parser_conditional_formatting(self, element):
-        rules = {}
         for cf in safe_iterator(element, '{%s}conditionalFormatting' % SHEET_MAIN_NS):
             if not cf.get('sqref'):
                 # Potentially flag - this attribute should always be present.
                 continue
             range_string = cf.get('sqref')
             cfRules = cf.findall('{%s}cfRule' % SHEET_MAIN_NS)
-            rules[range_string] = []
+            if range_string not in self.ws.conditional_formatting.parse_rules:
+                self.ws.conditional_formatting.parse_rules[range_string] = []
             for cfRule in cfRules:
                 if not cfRule.get('type') or cfRule.get('type') == 'dataBar':
                     # dataBar conditional formatting isn't supported, as it relies on the complex <extLst> tag
@@ -259,7 +216,10 @@ class WorkSheetParser(object):
                 rule = {'type': cfRule.get('type')}
                 for attr in ConditionalFormatting.rule_attributes:
                     if cfRule.get(attr) is not None:
-                        rule[attr] = cfRule.get(attr)
+                        if attr == 'priority':
+                            rule[attr] = int(cfRule.get(attr))
+                        else:
+                            rule[attr] = cfRule.get(attr)
 
                 formula = cfRule.findall('{%s}formula' % SHEET_MAIN_NS)
                 for f in formula:
@@ -280,19 +240,9 @@ class WorkSheetParser(object):
                         rule['colorScale']['cfvo'].append(cfvo)
                     colorNodes = colorScale.findall('{%s}color' % SHEET_MAIN_NS)
                     for color in colorNodes:
-                        c = Color(Color.BLACK)
-                        if self.color_index\
-                           and color.get('indexed') is not None\
-                           and 0 <= int(color.get('indexed')) < len(self.color_index):
-                            c.index = self.color_index[int(color.get('indexed'))]
-                        if color.get('theme') is not None:
-                            if color.get('tint') is not None:
-                                c.index = 'theme:%s:%s' % (color.get('theme'), color.get('tint'))
-                            else:
-                                c.index = 'theme:%s:' % color.get('theme')  # prefix color with theme
-                        elif color.get('rgb'):
-                            c.index = color.get('rgb')
-                        rule['colorScale']['color'].append(c)
+                        attrs = dict(color.items())
+                        color = Color(**attrs)
+                        rule['colorScale']['color'].append(color)
 
                 iconSet = cfRule.find('{%s}iconSet' % SHEET_MAIN_NS)
                 if iconSet is not None:
@@ -309,27 +259,36 @@ class WorkSheetParser(object):
                             cfvo['val'] = node.get('val')
                         rule['iconSet']['cfvo'].append(cfvo)
 
-                rules[range_string].append(rule)
-        if len(rules):
-            self.ws.conditional_formatting.setRules(rules)
+                self.ws.conditional_formatting.parse_rules[range_string].append(rule)
+
+    def parse_auto_filter(self, element):
+        self.ws.auto_filter.ref = element.get("ref")
+        for fc in safe_iterator(element, '{%s}filterColumn' % SHEET_MAIN_NS):
+            filters = fc.find('{%s}filters' % SHEET_MAIN_NS)
+            if filters is None:
+                continue
+            vals = [f.get("val") for f in safe_iterator(filters, '{%s}filter' % SHEET_MAIN_NS)]
+            blank = filters.get("blank")
+            self.ws.auto_filter.add_filter_column(fc.get("colId"), vals, blank=blank)
+        for sc in safe_iterator(element, '{%s}sortCondition' % SHEET_MAIN_NS):
+            self.ws.auto_filter.add_sort_condition(sc.get("ref"), sc.get("descending"))
 
 
-def fast_parse(ws, xml_source, string_table, style_table, color_index=None):
-
-    parser = WorkSheetParser(ws, xml_source, string_table, style_table, color_index)
+def fast_parse(ws, xml_source, shared_strings, style_table, color_index=None):
+    parser = WorkSheetParser(ws, xml_source, shared_strings, style_table, color_index)
     parser.parse()
+    del parser
 
 
-def read_worksheet(xml_source, parent, preset_title, string_table,
-                   style_table, color_index=None, sheet_codename=None, keep_vba=False):
+def read_worksheet(xml_source, parent, preset_title, shared_strings,
+                   style_table, color_index=None, worksheet_path=None, keep_vba=False):
     """Read an xml worksheet"""
-    if sheet_codename:
-        from openpyxl.reader.iter_worksheet import IterableWorksheet
-        ws = IterableWorksheet(parent, preset_title, sheet_codename,
-                               xml_source, string_table, style_table)
+    if worksheet_path:
+        ws = IterableWorksheet(parent, preset_title,
+                worksheet_path, xml_source, shared_strings, style_table)
     else:
         ws = Worksheet(parent, preset_title)
-        fast_parse(ws, xml_source, string_table, style_table, color_index)
+        fast_parse(ws, xml_source, shared_strings, style_table, color_index)
     if keep_vba:
         ws.xml_source = xml_source
     return ws
