@@ -7,17 +7,21 @@ from .hashable import HashableObject
 from openpyxl.descriptors import String, Bool, Float, MinMax, Integer, Alias, Set
 
 # Default Color Index as per 18.8.27 of ECMA Part 4
-COLOR_INDEX = ('00000000', '00FFFFFF', '00FF0000', '0000FF00', '000000FF',
-               '00FFFF00', '00FF00FF', '0000FFFF', '00000000', '00FFFFFF',
-               '00FF0000', '0000FF00', '000000FF', '00FFFF00', '00FF00FF', '0000FFFF',
-               '00800000', '00008000', '00000080', '00808000', '00800080', '00008080',
-               '00C0C0C0', '00808080', '009999FF', '00993366', '00FFFFCC', '00CCFFFF',
-               '00660066', '00FF8080', '000066CC', '00CCCCFF', '00000080', '00FF00FF',
-               '00FFFF00', '0000FFFF', '00800080', '00800000', '00008080', '000000FF',
-               '0000CCFF', '00CCFFFF', '00CCFFCC', '00FFFF99', '0099CCFF', '00FF99CC',
-               '00CC99FF', '00FFCC99', '003366FF', '0033CCCC', '0099CC00', '00FFCC00',
-               '00FF9900', '00FF6600', '00666699', '00969696', '00003366', '00339966',
-               '00003300', '00333300', '00993300', '00993366', '00333399', 'System Foreground', 'System Background')
+COLOR_INDEX = (
+    '00000000', '00FFFFFF', '00FF0000', '0000FF00', '000000FF', #0-4
+    '00FFFF00', '00FF00FF', '0000FFFF', '00000000', '00FFFFFF', #5-9
+    '00FF0000', '0000FF00', '000000FF', '00FFFF00', '00FF00FF', #10-14
+    '0000FFFF', '00800000', '00008000', '00000080', '00808000', #15-19
+    '00800080', '00008080', '00C0C0C0', '00808080', '009999FF', #20-24
+    '00993366', '00FFFFCC', '00CCFFFF', '00660066', '00FF8080', #25-29
+    '000066CC', '00CCCCFF', '00000080', '00FF00FF', '00FFFF00', #30-34
+    '0000FFFF', '00800080', '00800000', '00008080', '000000FF', #35-39
+    '0000CCFF', '00CCFFFF', '00CCFFCC', '00FFFF99', '0099CCFF', #40-44
+    '00FF99CC', '00CC99FF', '00FFCC99', '003366FF', '0033CCCC', #45-49
+    '0099CC00', '00FFCC00', '00FF9900', '00FF6600', '00666699', #50-54
+    '00969696', '00003366', '00339966', '00003300', '00333300', #55-59
+    '00993300', '00993366', '00333399', 'System Foreground', 'System Background' #60-64
+)
 
 BLACK = COLOR_INDEX[0]
 WHITE = COLOR_INDEX[1]
@@ -28,7 +32,7 @@ DARKBLUE = COLOR_INDEX[10]
 GREEN = COLOR_INDEX[3]
 DARKGREEN = COLOR_INDEX[9]
 YELLOW = COLOR_INDEX[5]
-DARKYELLOW = COLOR_INDEX[11]
+DARKYELLOW = COLOR_INDEX[19]
 
 
 class Color(HashableObject):
